import logging
from subprocess import Popen, PIPE, CREATE_NO_WINDOW, CalledProcessError

from util import sanitize_line

from const import *

class MinerMonitor():
	def __init__(self, cfg):
		self.pool_host = cfg["pool_host"]
		self.pool_port = cfg["pool_port"]
		self.pool_wallet = cfg["pool_wallet"]
		self.pool_worker = cfg["pool_worker"]

		self.process = None

		self.running = False

		self.last_recorded_hashrate = 0

		self.problem_zero_hashrate_detected = 0

	def construct_miner_command(self):
		return f".\miner\lolMiner.exe --algo ETHASH --pool {self.pool_host}:{self.pool_port} --user {self.pool_wallet}.{self.pool_worker} --watchdog exit --shortstats 1 --longstats 1"
	
	def start_miner(self):
		self.running = True
		self.watcher_loop()
	
	def stop_miner(self):
		if self.process: self.process.terminate()

		logging.info("Miner process killed")
		
		self.running = False

		self.last_recorded_hashrate = 0

	def watcher_loop(self):
		while self.running:
			self.problem_zero_hashrate_detected = 0

			logging.info("Starting miner process...")

			self.process = Popen(
				self.construct_miner_command(),
				stdout=PIPE,
				bufsize=1,
				universal_newlines=True,
				shell=False,
				creationflags=CREATE_NO_WINDOW
			)

			logging.info("Miner process started")

			for line in self.process.stdout:
				print(line, end="")

				clean_line = sanitize_line(line)

				if INFO_ANSI in line:
					logging.info(clean_line)
				elif WARNING_ANSI in line:
					logging.warning(clean_line)

				if clean_line.startswith("Average speed "):
					rate = float(clean_line.split(" ")[3])
					self.last_recorded_hashrate = rate

					if rate < MIN_AVERAGE_SPEED:
						self.problem_zero_hashrate_detected += 1

						if self.problem_zero_hashrate_detected >= LOW_HASHRATE_RESTART:
							logging.warning("Low hashrate detected, restarting process...")
							self.process.terminate()

				elif "Found a share of difficulty" in clean_line:
					diff_str = clean_line.split(" ")[7]
					diff = float(diff_str[:-1]) * METRIC_LEVELS[diff_str[-1]] / METRIC_LEVELS["G"]

					logging.info(clean_line)
				
				elif "DAG gen" in clean_line:
					logging.info(clean_line)

				elif clean_line.endswith("will be stopped."):
					logging.error("Mining error detected, restarting process...")
					self.process.terminate()
			
			if self.running:
				logging.error("Miner crashed, restarting process...")